#!/usr/bin/perl -w

# WHAT THIS SCRIPT DOES:
# clean (boilerplate stripping, discarding big/small pages, etc.),
# from the WaCky pipeline  
# INPUT:
# file produced by crawl_eng_pages.pl

use PotaModule; # PotaModule must be in @INC!!!
use Getopt::Std;

my $usage;
{
$usage = <<"_USAGE_";

process_webpages.pl -u target_urls downloaded_html_pages > out

This script is a wrapper to apply the filters in the PotaModule to
selected documents in a file generated with crawl_eng_pages.pl.

It is a slightly modified version of process_arc_files.pl, part of the WaCky
pipeline. I leave untouched as much code as possible...

Copyright 2005, Marco Baroni

This program is free software. You may copy or redistribute it under
the same terms as Perl itself.

_USAGE_
}
{
    my $blah = 1;
# this useless block is here because here document confuses
# emacs
}

%opts = ();

getopts('g:k:b:f:m:vh',\%opts);

if ($opts{h}) {
    print $usage;
    exit;
}

# read type/token maxima of acceptable bad words
# initialized to default values

$maxbadtypes = 3;
$maxbadtokens = 10;

# if user specified type/token maxima but no bad word list,
# something is wrong

if ($opts{f} && !($opts{b})){
    die "if you specify bad word thresholds, you should also have a bad word list";
}

if ($opts{f}) {
    ($maxbadtypes,$maxbadtokens) = split "/",$opts{f};
}

%bad = ();

if ($opts{b}) {
    open BADWORDS,$opts{b} or die "could not find bad word file $opts{b}";
    while (<BADWORDS>) {
	chomp;
	$bad{$_} = 1;
    }
    close BADWORDS;
}


# read type/token minima of required good words
# initialized to default values

$mingoodtypes = 10;
$mingoodtokens = 30;
$mingoodratio = 0.25;

# if user specified type/token minima but no good word list,
# something is wrong

if ($opts{k} && !($opts{g})){
    die "if you specify good word thresholds, you should also have a good word list";
}

if ($opts{k}) {
    ($mingoodtypes,$mingoodtokens,$mingoodratio) = split "/",$opts{k};
}

%good = ();

if ($opts{g}) {
    open GOODWORDS,$opts{g} or die "could not find good word file $opts{g}";
    while (<GOODWORDS>) {
	chomp;
	$good{$_} = 1;
    }
    close GOODWORDS;
}
else { # required minima should be set to 0;
    $mingoodtypes = 0;
    $mingoodtokens = 0;
    $mingoodratio = 0;
}

# maximum/minimum size of doc before processing
# defaults to 5Kb/200Kb
# (or equivalent char numbers in "character semantics")

$minbytesize = 5119;
$maxbytesize = 204801;

if ($opts{m}) {
    ($minbytesize,$maxbytesize) = split "/",$opts{m};
}



# go through the file. I leave the code untouched, although
# I suspect much of it is useless for the present purposes
 
$in_good_page = 0;
while (<>) {

    chomp;

    if ($_=~/^CURRENT_URL[ \t]/) {
	 
      (undef, $url,$unihome,$uniname,$unirank,$unicountry,$langfam,$countrystatus,$langstatus,undef,$pagelevel) = split "[ \t]+",$_;

	# first, if there is a previous file, we should process it
	if ($in_good_page) {

	    $potaref = dig_content($text,\%bad,
				   $maxbadtypes,$maxbadtokens,
				   \%good,$mingoodtypes,
				   $mingoodtokens,$mingoodratio,
				   $minbytesize,$maxbytesize);

	    # if we did get something back from the cleaning process,
	    # let's print it
	    if (${$potaref}[0]) {
		print "CURRENT_URL $prev_url\t$prev_unihome\t$prev_uniname\t$prev_unirank\t$prev_unicountry\t$prev_langfam\t$prev_countrystatus\t$prev_langstatus\t$prev_pagelevel\n";
		print ${$potaref}[0],"\n";
	    }
	
	    # reset values
	    $in_good_page = 0;
	    $part_to_print = 0;
	    $text = "";
	    
	}

	# now, let's see if this is a potentially interesting page

	    if (($url)) {
	    $in_good_page = 1;
	    $prev_url = $url;
	    $prev_unihome = $unihome;
	    $prev_uniname = $uniname;
	    $prev_unirank = $unirank;
	    $prev_unicountry = $unicountry;
	    $prev_langfam = $langfam;
	    $prev_countrystatus = $countrystatus;
	    $prev_langstatus = $langstatus;
	    $prev_pagelevel = $pagelevel;
	}
	next;
    }

    # if we are in good page, 
    # we must move on until we see the first blank like
    if (($in_good_page) && (!$part_to_print)) {
	if (/^\s*$/) {
	    $part_to_print = 1;
	}
	next;
    }


    # at this point, we should be dealing with contents, so we
    # concatenate if we are in good page
    if ($part_to_print) {
	$text .= "$_\n";
    }
}

# if very last page happened to be a good one, we have to process it
# as well

if ($in_good_page) {

    $potaref = dig_content($text,\%bad,
			   $maxbadtypes,$maxbadtokens,
			   \%good,$mingoodtypes,
			   $mingoodtokens,$mingoodratio,
			   $minbytesize,$maxbytesize);
    
    # if we did get something back from the cleaning process,
    # let's print it
    if (${$potaref}[0]) {
      print "CURRENT_URL $prev_url\t$prev_unihome\t$prev_uniname\t$prev_unirank\t$prev_unicountry\n";
      print ${$potaref}[0],"\n";
    }

}

print STDERR "\nDone!\n";

=head1 DESCRIPTION

ALL THAT FOLLOWS IS FROM THE ORIGINAL WACKY SCRIPT

This is a simple script that applies the filters in the PotaModule to
a list of documents (to be specified with option -u) that are
retrieved from the contents of one or more Heritrix ARC files (since
these are gzipped, you should pass them, e.g., through zcat before
piping them to this script). ARC files are files that store the
documents retrieved by a crawl of the Internet Archive Heritrix
crawler (see url below).

Of course, the script needs the PotaModule to be installed on the
machine you are using (by "installed", I mean that the PotaModule is
somewhere in @INC, so that perl will see it).

To see what the PotaModule does, type:

perldoc PotaModule

(or use the appropriate path, depending on where the PotaModule lives
-- for example, I must type: perldoc MyModules::PotaModule).

The script outputs the cleaned contents of all the documents in the
specified list that were found in the archive(s), and that passed all
the filtering stages.

=head2 ARGUMENT AND OPTIONS

The only argument to the script is a file or STDIN stream with the
contents of one or more ARC files.

The options are:

B<-h>: Prints short info about the script and quits.

B<-v>: A very quiet verbose option, that is simply sending the name of
the ARC file currently being processed to standard error (so that we
know where we are at when processing a lot of data).

B<-u filename>: This is not really an "option", as it is mandatory: a list of
the urls corresponding to the documents of interest, in
one-url-per-line format. The idea is that the ARC files will contain
plenty of documents that you are not interested in processing (e.g.,
perfect duplicates, robots.txt files, error messages), so you
pre-compile a list of documents that you are likely to be interested
in.

B<-b filename>: A list of "bad" words (e.g., pornographic terms). If a
document contains more than a certain number of types or tokens from
this list, it will not be printed (see option B<-f> for default
thresholds and how to change them).

B<-f type_N/token_N>: Two slash-separated values that specify,
respectively, the number of types and tokens from the "bad" word list
sufficient to cause a document to be discarded. Default thresholds are
3 types and 10 tokens. Of course, this option can only be specified if
a bad word list is also passed.

B<-g filename>: A list of "good" words (e.g., function words). A
document is printed only if it contains a certain number of types and
tokens from this list, and if the ratio of tokens from the list to
total token is above a certain threshold (see option B<-k> for default
thresholds and how to change them).

B<-k type_N/token_N/ratio>: Three slash-separated values that specify,
respectively, the minimum number of types and tokens from the "good"
word list that a document must contain to be printed, and the minimum
ratio of tokens from the list to total tokens in the document that a
document must have to be printed. Default thresholds are 10 types, 30
tokens and a ratio of 0.25 (or 0, 0 and 0 if no "good" word list is
specified). Of course, the option can only be used if a good word list
is also passed.

B<-m min_size/max_size>: Two slash-separated values specifying the
minimum and maximum allowed sizes (in characters) for documents to be
printed. Default values are 5119 and 204801, respectively, which, in
single-byte encodings, means 5KB and 2000KB, respectively.

=head1 DEPENDENCIES

PotaModule

=head1 AUTHOR

Marco Baroni, baroni AT sslmit.unibo.it

=head1 ACKNOWLEDGMENTS

Thanks to Eros Zanchetta for help, advice and testing.

=head1 BUGS

The current regular expressions to find documents inside ARC files
will break down if the ARC file format changes, and in any case on
January 1 2010 (the Y2K10 bug ;-) ).

Probably many others: if you find one, please let me know: baroni AT
sslmit unibo it


=head1 COPYRIGHT

Copyright 2005, Marco Baroni

This program is free software. You may copy or redistribute it under
the same terms as Perl itself.

=cut
